
/* Copyright (C) 1992  AHPCRC, Univeristy of Minnesota
 * Earlier versions Copyright 1992,  Minnesota Supercomputer Center Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named 'Copying'; if not, write to
 * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139.
 */

/* Author:
 *	Ken Chin-Purcell (ken@ahpcrc.umn.edu)
 *	Army High Performance Computing Research Center (AHPCRC)
 *	Univeristy of Minnesota
 *
 * $Header: /usr/people/ken/gvl/bob/RCS/vox.h,v 2.2 92/10/27 16:50:33 ken Exp Locker: ken $
 *
 * $Log:	vox.h,v $
 * Revision 2.2  92/10/27  16:50:33  ken
 * Added basic point support
 * 
 * Revision 2.1  92/10/19  17:03:41  ken
 * *** empty log message ***
 * 
 * Revision 1.1  92/10/19  17:03:04  ken
 * Initial revision
 * 
 */

/* Prototypes */

/* vox.c */
void	SetView (int width, int height);
void	SetupGL (void);
void	ReadAllPoint (void);
void	ReadPoint (void);
void	ReadData (void);
void	PackColor (unsigned char *cmap);
void	PackAlpha (unsigned char *amap);
void	DrawVoxelWindow (unsigned fast, unsigned save);
void	DrawStereoWindow (unsigned fast);
void	DrawFinderWindow (void);
void	DrawFinderSpecial (FinderEnum mode, unsigned plane);
