"Icol.fileFormat: asciiIndex",
"Icol.showCopyright: false",
"*background: grey75",
"*allowShellResize: True",
"*fontList: -*-helvetica-medium-r-normal--*-100-*-*-*-*-*-*",
"*XmPushButton.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmPushButtonGadget.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*menuBar*fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*main*rgbForm.XmLabel.fontList: -*-helvetica-bold-o-normal--*-100-*-*-*-*-*-*",
"*main*rgbForm.XmScale.fontList: 7x13",
"*XmMenuShell.visual: PseudoColor",
"*XmDialogShell.visual: PseudoColor",
"*fsBox.dialogTitle: Icol File Select",
"*fsBox*XmText.width: 400",
"*fsBox*radioLabel.labelString: Format",
"*fsBox*radioLabel.alignment: ALIGNMENT_BEGINNING",
"*fsBox*radioLabel.topAttachment: ATTACH_FORM",
"*fsBox*radioLabel.leftAttachment: ATTACH_FORM",
"*fsBox*radioLabel.rightAttachment: ATTACH_FORM",
"*fsBox*radioFrame.shadowType: SHADOW_ETCHED_IN",
"*fsBox*radioFrame.topAttachment: ATTACH_WIDGET",
"*fsBox*radioFrame.topWidget: radioLabel",
"*fsBox*radioFrame.leftAttachment: ATTACH_FORM",
"*fsBox*radioFrame.rightAttachment: ATTACH_FORM",
"*fsBox*radio.spacing: 0",
"*fsBox*XmToggleButtonGadget.selectColor: red",
"*fsBox*asciiIndex.labelString: Indexed Ascii Text",
"*fsBox*asciiIndex.set: true",
"*fsBox*asciiPlain.labelString: Non-indexed Ascii Text",
"*fsBox*binLeave.labelString: Interleaved Binary (rgbrgbrgb...)",
"*fsBox*binRow.labelString: Non-interleaved Binary (rrr...ggg...bbb...)",
"*quitBox.dialogTitle: Icol Quit",
"*quitBox.messageString: Save colorbar changes?",
"*quitBox.okLabelString: Save",
"*quitBox.cancelLabelString: Discard",
"*copyright.dialogTitle: All About Icol",
"*copyright*fontList: -*-itc zapf chancery-*-*-*--*-140-*-*-*-*-*-*",
"*copyright*background: red4",
"*copyright*foreground: gold",
"*copyright.okLabelString: OK",
"*copyright.cancelLabelString: Mail Comment",
"*copyright.messageString: \\nIcol  -  Interpolated Color\\n   Version 1.1\\n      Copyright 1992 University of Minnesota\\n      Distributed under the GNU General Public License\\n\\nFrom:\\n   Graphics and Visualization Lab,\\n      Army High Performance Computing Research Center,\\n      University of Minnesota\\n\\nWritten by:\\n   Ken Chin-Purcell\\n",
"*bugBox.dialogTitle: Icol Comment",
"*bugBox.horizontalSpacing: 6",
"*bugBox.verticalSpacing: 6",
"*bugBox.send.labelString: Send",
"*bugBox.send.showAsDefault: true",
"*bugBox.send.marginWidth: 6",
"*bugBox.send.marginHeight: 4",
"*bugBox.send.leftAttachment: ATTACH_FORM",
"*bugBox.send.bottomAttachment: ATTACH_FORM",
"*bugBox.cancel.labelString: Cancel",
"*bugBox.cancel.marginWidth: 6",
"*bugBox.cancel.marginHeight: 4",
"*bugBox.cancel.rightAttachment: ATTACH_FORM",
"*bugBox.cancel.bottomAttachment: ATTACH_FORM",
"*bugBox.cancel.bottomOffset: 12",
"*bugBox.sep.bottomAttachment: ATTACH_WIDGET",
"*bugBox.sep.bottomWidget: send",
"*bugBox.sep.leftAttachment: ATTACH_FORM",
"*bugBox.sep.rightAttachment: ATTACH_FORM",
"*bugBox.bugText.value: My comment on 'icol' is...\\n\\n",
"*bugBox.bugText.editMode: MULTI_LINE_EDIT",
"*bugBox.bugText.width: 400",
"*bugBox.bugText.height: 300",
"*bugBox.bugText.topAttachment: ATTACH_FORM",
"*bugBox.bugText.leftAttachment: ATTACH_FORM",
"*bugBox.bugText.rightAttachment: ATTACH_FORM",
"*bugBox.bugText.bottomAttachment: ATTACH_WIDGET",
"*bugBox.bugText.bottomWidget: sep",
"*fileError.dialogTitle: Icol File Error",
"*fileError.okLabelString: Try Again",
"*main*menuBar*background: grey65",
"*menuBar*XmToggleButton.selectColor: red",
"*menuBar*XmToggleButton.visibleWhenOff: true",
"*menuBar*XmToggleButton.marginHeight: 0",
"*menuBar*XmToggleButton.marginTop: 4",
"*menuBar*XmToggleButton.marginBottom: 3",
"*menuBar.topAttachment: ATTACH_FORM",
"*menuBar.leftAttachment: ATTACH_FORM",
"*menuBar.rightAttachment: ATTACH_FORM",
"*sepMenu.separatorType: Shadow_Etched_In",
"*control.labelString: File",
"*control.mnemonic: F",
"*controlMenu.about.labelString: About Icol...",
"*controlMenu.about.mnemonic: A",
"*controlMenu.open.labelString: Open...",
"*controlMenu.open.mnemonic: O",
"*controlMenu.save.labelString: Save...",
"*controlMenu.save.mnemonic: S",
"*controlMenu.geti.labelString: Get Colormap Property",
"*controlMenu.geti.mnemonic: G",
"*controlMenu.geti.accelerator: Ctrl<Key>g:",
"*controlMenu.geti.acceleratorText: ^g",
"*controlMenu.seti.labelString: Set Colormap Property",
"*controlMenu.seti.mnemonic: t",
"*controlMenu.seti.accelerator: Ctrl<Key>s:",
"*controlMenu.seti.acceleratorText: ^s",
"*controlMenu.grabColor.labelString: Get X Colormap",
"*controlMenu.grabColor.accelerator: Shift Ctrl<Key>g:",
"*controlMenu.grabColor.acceleratorText: ^G",
"*controlMenu.setColor.labelString: Set X Colormap",
"*controlMenu.setColor.accelerator: Shift Ctrl<Key>s:",
"*controlMenu.setColor.acceleratorText: ^S",
"*controlMenu.pickColor.labelString: Pick Color",
"*controlMenu.pickColor.mnemonic: P",
"*controlMenu.pickColor.accelerator: Ctrl<Key>p:",
"*controlMenu.pickColor.acceleratorText: ^p",
"*controlMenu.bug.labelString: Bug Report",
"*controlMenu.bug.mnemonic: B",
"*controlMenu.quit.labelString: Exit",
"*controlMenu.quit.mnemonic: x",
"*controlMenu.quit.accelerator: Ctrl<Key>x:",
"*controlMenu.quit.acceleratorText: ^x",
"*edit.labelString: Edit",
"*edit.mnemonic: E",
"*editMenu.undo.labelString: Undo",
"*editMenu.undo.mnemonic: U",
"*editMenu.undo.accelerator: Ctrl<Key>u:",
"*editMenu.undo.acceleratorText: ^u",
"*editMenu.redo.labelString: Redo",
"*editMenu.redo.mnemonic: R",
"*editMenu.redo.accelerator: Ctrl<Key>d:",
"*editMenu.redo.acceleratorText: ^d",
"*editMenu.snap.labelString: Snap",
"*editMenu.snap.mnemonic: S",
"*editMenu.snap.accelerator: Ctrl<Key>n:",
"*editMenu.snap.acceleratorText: ^n",
"*editMenu.snapAll.labelString: Snap All",
"*editMenu.snapAll.mnemonic: A",
"*editMenu.snapAll.accelerator: Ctrl<Key>a:",
"*editMenu.snapAll.acceleratorText: ^a",
"*editMenu.placeKnot.labelString: Place Knots",
"*editMenu.placeKnot.mnemonic: P",
"*editMenu.placeKnot.accelerator: Ctrl<Key>k:",
"*editMenu.placeKnot.acceleratorText: ^k",
"*editMenu.snap1.labelString: Snap Red",
"*editMenu.snap1.accelerator: Ctrl<Key>1:",
"*editMenu.snap1.acceleratorText: ^1",
"*editMenu.snap1.foreground: #d00",
"*editMenu.snap2.labelString: Snap Green",
"*editMenu.snap2.accelerator: Ctrl<Key>2:",
"*editMenu.snap2.acceleratorText: ^2",
"*editMenu.snap2.foreground: #070",
"*editMenu.snap3.labelString: Snap Blue",
"*editMenu.snap3.accelerator: Ctrl<Key>3:",
"*editMenu.snap3.acceleratorText: ^3",
"*editMenu.snap3.foreground: #00b",
"*mode.labelString: Mode",
"*mode.mnemonic: M",
"*modeMenu.rgb.labelString: Red / Green / Blue",
"*modeMenu.rgb.mnemonic: R",
"*modeMenu.rgb.indicatorType: ONE_OF_MANY",
"*modeMenu.rgb.set: true",
"*modeMenu.rgb.accelerator: Ctrl<Key>r:",
"*modeMenu.rgb.acceleratorText: ^r",
"*modeMenu.hsv.labelString: Hue / Saturation / Value",
"*modeMenu.hsv.mnemonic: H",
"*modeMenu.hsv.indicatorType: ONE_OF_MANY",
"*modeMenu.hsv.accelerator: Ctrl<Key>h:",
"*modeMenu.hsv.acceleratorText: ^h",
"*modeMenu.yuv.labelString: Y / U / V  (Video)",
"*modeMenu.yuv.mnemonic: Y",
"*modeMenu.yuv.indicatorType: ONE_OF_MANY",
"*modeMenu.yuv.accelerator: Ctrl<Key>y:",
"*modeMenu.yuv.acceleratorText: ^y",
"*modeMenu.alpha.labelString: Alpha only",
"*modeMenu.alpha.mnemonic: A",
"*modeMenu.alpha.indicatorType: ONE_OF_MANY",
"*modeMenu.auto.labelString: Auto Save",
"*modeMenu.auto.mnemonic: S",
"*modeMenu.auto.indicatorType: N_OF_MANY",
"*modeMenu.autoi.labelString: Auto Set Property",
"*modeMenu.autoi.mnemonic: P",
"*modeMenu.autoi.indicatorType: N_OF_MANY",
"*modeMenu.autoi.set: true",
"*modeMenu.stomp.labelString: Stomp",
"*modeMenu.stomp.mnemonic: t",
"*modeMenu.stomp.indicatorType: N_OF_MANY",
"*modeMenu.stomp.accelerator: Ctrl<Key>t:",
"*modeMenu.stomp.acceleratorText: ^t",
"*brush.labelString: Brush",
"*brush.mnemonic: B",
"*brushMenu.square.labelType: PIXMAP",
"*brushMenu.square.indicatorType: ONE_OF_MANY",
"*brushMenu.square.set: true",
"*brushMenu.circle.labelType: PIXMAP",
"*brushMenu.circle.indicatorType: ONE_OF_MANY",
"*brushMenu.diamond.labelType: PIXMAP",
"*brushMenu.diamond.indicatorType: ONE_OF_MANY",
"*ahpcrc.labelString: AHPCRC Interpolate Color",
"*ahpcrc.foreground: red4",
"*ahpcrc.fontList: -*-times-bold-r-*-*-*-140-*-*-*-*-*-*",
"*ahpcrc.topAttachment: ATTACH_WIDGET",
"*ahpcrc.topWidget: menuBar",
"*ahpcrc.topOffset: 6",
"*ahpcrc.leftAttachment: ATTACH_FORM",
"*ahpcrc.leftOffset: 6",
"*ahpcrc.rightAttachment: ATTACH_FORM",
"*ahpcrc.rightOffset: 6",
"*ahpcrc.highlightThickness: 0",
"*pane.marginHeight: 6",
"*pane.marginWidth: 6",
"*pane.spacing: 16",
"*pane.topAttachment: ATTACH_WIDGET",
"*pane.topWidget: ahpcrc",
"*pane.leftAttachment: ATTACH_FORM",
"*pane.rightAttachment: ATTACH_FORM",
"*pane.bottomAttachment: ATTACH_WIDGET",
"*pane.bottomWidget: paneSep",
"*pane.XmFrame.paneMinimum: 80",
"*chartFrame.shadowType: SHADOW_IN",
"*chart.height: 160",
"*chart*background: #858585",
"*buttonFrame.shadowType: SHADOW_IN",
"*button.width: 300",
"*button.height: 301",
"*button*background: #858585",
"*paneSep.separatorType: Shadow_Etched_In",
"*paneSep.leftAttachment: ATTACH_FORM",
"*paneSep.rightAttachment: ATTACH_FORM",
"*paneSep.bottomAttachment: ATTACH_WIDGET",
"*paneSep.bottomWidget: rgbForm",
"*rgbForm.skipAdjust: true",
"*rgbForm.leftAttachment: ATTACH_FORM",
"*rgbForm.rightAttachment: ATTACH_FORM",
"*rgbForm.bottomAttachment: ATTACH_FORM",
"*rgbForm.XmLabel.leftAttachment: ATTACH_FORM",
"*rgbForm.XmLabel.alignment: ALIGNMENT_BEGINNING",
"*rgbForm.XmLabel.leftOffset: 4",
"*rgbForm.XmLabel.topAttachment: ATTACH_WIDGET",
"*rgbForm.XmLabel.topOffset: 11",
"*rgbForm.XmLabel.bottomOffset: 4",
"*rgbForm.XmLabel.width: 52",
"*rgbForm.cellLabel.labelString: Color Cell 100",
"*rgbForm.cellLabel.topAttachment: ATTACH_FORM",
"*rgbForm.cellLabel.topOffset: 4",
"*rgbForm.cellLabel.rightAttachment: ATTACH_FORM",
"*rgbForm.redLabel.labelString: Red",
"*rgbForm.redLabel.topWidget: cellLabel",
"*rgbForm.redLabel.foreground: #d00",
"*rgbForm.grnLabel.labelString: Green",
"*rgbForm.grnLabel.topWidget: redLabel",
"*rgbForm.grnLabel.foreground: #070",
"*rgbForm.bluLabel.labelString: Blue",
"*rgbForm.bluLabel.topWidget: grnLabel",
"*rgbForm.bluLabel.foreground: #00b",
"*rgbForm.bluLabel.bottomAttachment: ATTACH_FORM",
"*rgbForm.XmScale.showValue: on",
"*rgbForm.XmScale.bottomAttachment: ATTACH_OPPOSITE_WIDGET",
"*rgbForm.XmScale.rightAttachment: ATTACH_FORM",
"*rgbForm.XmScale.rightOffset: 4",
"*rgbForm.XmScale.leftAttachment: ATTACH_FORM",
"*rgbForm.XmScale.leftOffset: 58",
"*rgbForm.XmScale.orientation: HORIZONTAL",
"*rgbForm.XmScale.minimum: 0",
"*rgbForm.XmScale.maximum: 255",
"*rgbForm.redScale.bottomWidget: redLabel",
"*rgbForm.grnScale.bottomWidget: grnLabel",
"*rgbForm.bluScale.bottomWidget: bluLabel",
