"XRaz.geometry: +100+300",
"*background: grey75",
"*allowShellResize: true",
"*fontList: -*-helvetica-medium-r-normal--*-100-*-*-*-*-*-*",
"*XmPushButton.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmPushButtonGadget.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*menuBar*fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmLabel.fontList: -*-helvetica-bold-o-normal--*-100-*-*-*-*-*-*",
"*XmToggleButton.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmToggleButton.selectColor: red",
"*XmToggleButton.visibleWhenOff: true",
"*XmToggleButtonGadget.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmToggleButtonGadget.selectColor: red",
"*XmToggleButtonGadget.visibleWhenOff: true",
"*XmTextField.background: grey82",
"*XmText.background: grey82",
"*fsBox.dialogTitle: Xraz Script Select",
"*fsBox*XmText.width: 400",
"*fsImage.dialogTitle: Xraz Image Files",
"*fsImage.filterLabelString: File Pattern:",
"*fsImage*XmText.width: 400",
"*rawSelect.dialogTitle: Raw Disk Parameters",
"*rawSelect.dialogStyle: DIALOG_APPLICATION_MODAL",
"*rawSelect.horizontalSpacing: 4",
"*rawSelect.verticalSpacing: 4",
"*rawSelect.XmTextField.fontList: 7x13",
"*rawSelect.XmTextField.columns: 25",
"*rawSelect.XmTextField.leftAttachment: Attach_Form",
"*rawSelect.XmTextField.leftOffset: 90",
"*rawSelect.XmTextField.rightAttachment: Attach_Form",
"*rawSelect.XmTextField.topAttachment: Attach_Widget",
"*rawSelect.XmLabel.leftAttachment: Attach_Form",
"*rawSelect.XmLabel.bottomAttachment: Attach_Opposite_Widget",
"*rawSelect.XmLabel.bottomOffset: 4",
"*rawSelect.rawRC.packing: Pack_Column",
"*rawSelect.rawRC.topAttachment: Attach_Form",
"*rawSelect.rawRC.leftAttachment: Attach_Form",
"*rawSelect.rawRC.leftOffset: 67",
"*rawSelect.rawRC.buttonCount: 5",
"*rawSelect.rawRC.buttons: /dev/anim0, /dev/anim1, /dev/anim2, 					/dev/anim3, /dev/anim4",
"*rawSelect.rawRC.buttonSet: 0",
"*rawSelect.rawname.value: /dev/anim0",
"*rawSelect.rawname.topWidget: rawRC",
"*rawSelect.rawlabel.labelString: Disk",
"*rawSelect.rawlabel.bottomWidget: rawname",
"*rawSelect.offset.value: 0",
"*rawSelect.offset.topWidget: rawname",
"*rawSelect.offlabel.labelString: Offset",
"*rawSelect.offlabel.bottomWidget: offset",
"*rawSelect.numraw.value: 1000",
"*rawSelect.numraw.topWidget: offset",
"*rawSelect.numlabel.labelString: Frames",
"*rawSelect.numlabel.bottomWidget: numraw",
"*rawSelect.blocksize.value: 4k",
"*rawSelect.blocksize.topWidget: numraw",
"*rawSelect.blocklabel.labelString: Block Size",
"*rawSelect.blocklabel.bottomWidget: blocksize",
"*rawSelect.chunksize.value: 4k",
"*rawSelect.chunksize.leftAttachment: Attach_Form",
"*rawSelect.chunksize.leftOffset: 90",
"*rawSelect.chunksize.rightAttachment: Attach_Form",
"*rawSelect.chunksize.topAttachment: Attach_Widget",
"*rawSelect.chunksize.topWidget: blocksize",
"*rawSelect.chunksize.topOffset: -2",
"*rawSelect.chunksize.minimum: 1",
"*rawSelect.chunksize.maximum: 32",
"*rawSelect.chunksize.value: 1",
"*rawSelect.chunksize.showValue: on",
"*rawSelect.chunksize.orientation: Horizontal",
"*rawSelect.chunksize.fontList: 7x13",
"*rawSelect.chunklabel.labelString: Chunk Size",
"*rawSelect.chunklabel.bottomWidget: chunksize",
"*rawSelect.chunklabel.bottomOffset: 0",
"*rawSelect.dsep2.topAttachment: Attach_Widget",
"*rawSelect.dsep2.topWidget: chunksize",
"*rawSelect.dsep2.topOffset: 10",
"*rawSelect.dsep2.leftAttachment: Attach_Form",
"*rawSelect.dsep2.rightAttachment: Attach_Form",
"*rawSelect.rawok.labelString: OK",
"*rawSelect.rawok.width: 70",
"*rawSelect.rawok.topAttachment: Attach_Widget",
"*rawSelect.rawok.topWidget: dsep2",
"*rawSelect.rawok.topOffset: 10",
"*rawSelect.rawok.leftAttachment: Attach_Form",
"*rawSelect.rawok.leftOffset: 30",
"*rawSelect.rawcancel.labelString: Cancel",
"*rawSelect.rawcancel.width: 70",
"*rawSelect.rawcancel.topAttachment: Attach_Widget",
"*rawSelect.rawcancel.topWidget: dsep2",
"*rawSelect.rawcancel.topOffset: 10",
"*rawSelect.rawcancel.rightAttachment: Attach_Form",
"*rawSelect.rawcancel.rightOffset: 30",
"*rawSelect.dsep3.separatorType: No_Line",
"*rawSelect.dsep3.orientation: Horizontal",
"*rawSelect.dsep3.rightAttachment: Attach_Form",
"*rawSelect.dsep3.leftAttachment: Attach_Form",
"*rawSelect.dsep3.topAttachment: Attach_Widget",
"*rawSelect.dsep3.topWidget: rawok",
"*rawSelect.dsep3.topOffset: 10",
"*formatSelect.dialogTitle: File Format",
"*formatSelect.dialogStyle: DIALOG_APPLICATION_MODAL",
"*formatSelect.horizontalSpacing: 4",
"*formatSelect.verticalSpacing: 4",
"*formatSelect.XmTextField.fontList: 7x13",
"*formatSelect.XmTextField.columns: 30",
"*formatSelect.XmTextField.leftAttachment: Attach_Form",
"*formatSelect.XmTextField.leftOffset: 100",
"*formatSelect.XmTextField.rightAttachment: Attach_Form",
"*formatSelect.XmTextField.topAttachment: Attach_Widget",
"*formatSelect.XmLabel.leftAttachment: Attach_Form",
"*formatSelect.XmLabel.bottomAttachment: Attach_Opposite_Widget",
"*formatSelect.XmLabel.bottomOffset: 4",
"*formatSelect.formatRC.packing: Pack_Column",
"*formatSelect.formatRC.topAttachment: Attach_Form",
"*formatSelect.formatRC.leftAttachment: Attach_Form",
"*formatSelect.formatRC.leftOffset: 77",
"*formatSelect.formatRC.buttonCount: 3",
"*formatSelect.formatRC.buttons: 8 bit (pseudo8), 					24 bit interleaved (rgbbyte), 					24 bit by planes (rgbplane)",
"*formatSelect.formatRC.buttonSet: 0",
"*formatSelect.fsep1.topAttachment: Attach_Widget",
"*formatSelect.fsep1.topWidget: formatRC",
"*formatSelect.fsep1.leftAttachment: Attach_Form",
"*formatSelect.fsep1.rightAttachment: Attach_Form",
"*formatSelect.filex.value: 1024",
"*formatSelect.filex.topWidget: fsep1",
"*formatSelect.filex.topOffset: 10",
"*formatSelect.fxlabel.labelString: X dimension",
"*formatSelect.fxlabel.bottomWidget: filex",
"*formatSelect.filey.value: 1024",
"*formatSelect.filey.topWidget: filex",
"*formatSelect.fylabel.labelString: Y dimension",
"*formatSelect.fylabel.bottomWidget: filey",
"*formatSelect.fsizeRC.packing: Pack_Column",
"*formatSelect.fsizeRC.topAttachment: Attach_Widget",
"*formatSelect.fsizeRC.topWidget: filey",
"*formatSelect.fsizeRC.leftAttachment: Attach_Form",
"*formatSelect.fsizeRC.leftOffset: 77",
"*formatSelect.fsizeRC.buttonCount: 5",
"*formatSelect.fsizeRC.buttons: 1024 by 1024, 					512 by 512, 					256 by 256, 					640 by 480, 					720 by 486",
"*formatSelect.fsizeRC.buttonSet: 0",
"*formatSelect.fsep2.topAttachment: Attach_Widget",
"*formatSelect.fsep2.topWidget: fsizeRC",
"*formatSelect.fsep2.topOffset: 8",
"*formatSelect.fsep2.leftAttachment: Attach_Form",
"*formatSelect.fsep2.rightAttachment: Attach_Form",
"*formatSelect.formatok.labelString: OK",
"*formatSelect.formatok.width: 70",
"*formatSelect.formatok.topAttachment: Attach_Widget",
"*formatSelect.formatok.topWidget: fsep2",
"*formatSelect.formatok.topOffset: 10",
"*formatSelect.formatok.leftAttachment: Attach_Form",
"*formatSelect.formatok.leftOffset: 30",
"*formatSelect.formatcancel.labelString: Cancel",
"*formatSelect.formatcancel.width: 70",
"*formatSelect.formatcancel.topAttachment: Attach_Widget",
"*formatSelect.formatcancel.topWidget: fsep2",
"*formatSelect.formatcancel.topOffset: 10",
"*formatSelect.formatcancel.rightAttachment: Attach_Form",
"*formatSelect.formatcancel.rightOffset: 30",
"*formatSelect.fsep3.separatorType: No_Line",
"*formatSelect.fsep3.orientation: Horizontal",
"*formatSelect.fsep3.rightAttachment: Attach_Form",
"*formatSelect.fsep3.leftAttachment: Attach_Form",
"*formatSelect.fsep3.topAttachment: Attach_Widget",
"*formatSelect.fsep3.topWidget: formatok",
"*formatSelect.fsep3.topOffset: 10",
"*copyright.dialogTitle: Xraz Info",
"*copyright*fontList: -*-itc zapf chancery-*-*-*--*-140-*-*-*-*-*-*",
"*copyright*background: red4",
"*copyright*foreground: gold",
"*copyright.okLabelString: Enough About Xraz",
"*copyright.cancelLabelString: Mail Comment",
"*copyright.messageString: \\nXraz  -  X interface to Raz, a disk based raster viewer.\\nVersion  1.1\\n      Copyright 1992 University of Minnesota\\n      Distributed under the GNU General Public License\\n\\nFrom:\\n   Graphics and Visualization Lab,\\n      Army High Performance Computing Research Center,\\n      University of Minnesota\\n\\nWritten by:\\n   Ken Chin-Purcell\\n",
"*bugBox.dialogTitle: Xraz Comment",
"*bugBox.horizontalSpacing: 6",
"*bugBox.verticalSpacing: 6",
"*bugBox.bugsend.labelString: Send",
"*bugBox.bugsend.showAsDefault: true",
"*bugBox.bugsend.marginWidth: 6",
"*bugBox.bugsend.marginHeight: 4",
"*bugBox.bugsend.leftAttachment: ATTACH_FORM",
"*bugBox.bugsend.bottomAttachment: ATTACH_FORM",
"*bugBox.bugcancel.labelString: Cancel",
"*bugBox.bugcancel.marginWidth: 6",
"*bugBox.bugcancel.marginHeight: 4",
"*bugBox.bugcancel.rightAttachment: ATTACH_FORM",
"*bugBox.bugcancel.bottomAttachment: ATTACH_FORM",
"*bugBox.bugcancel.bottomOffset: 12",
"*bugBox.sep.bottomAttachment: ATTACH_WIDGET",
"*bugBox.sep.bottomWidget: bugsend",
"*bugBox.sep.leftAttachment: ATTACH_FORM",
"*bugBox.sep.rightAttachment: ATTACH_FORM",
"*bugBox.bugText.value: My comment on Xraz is...\\n\\n",
"*bugBox.bugText.editMode: MULTI_LINE_EDIT",
"*bugBox.bugText.width: 400",
"*bugBox.bugText.height: 300",
"*bugBox.bugText.topAttachment: ATTACH_FORM",
"*bugBox.bugText.leftAttachment: ATTACH_FORM",
"*bugBox.bugText.rightAttachment: ATTACH_FORM",
"*bugBox.bugText.bottomAttachment: ATTACH_WIDGET",
"*bugBox.bugText.bottomWidget: sep",
"*scriptBox.dialogTitle: Xraz Script",
"*scriptBox.horizontalSpacing: 6",
"*scriptBox.verticalSpacing: 6",
"*scriptBox.autoUnmanage: False",
"*scriptBox.scriptsend.labelString: Send",
"*scriptBox.scriptsend.showAsDefault: true",
"*scriptBox.scriptsend.marginWidth: 6",
"*scriptBox.scriptsend.marginHeight: 4",
"*scriptBox.scriptsend.leftAttachment: ATTACH_FORM",
"*scriptBox.scriptsend.bottomAttachment: ATTACH_FORM",
"*scriptBox.scriptdone.labelString: Done",
"*scriptBox.scriptdone.marginWidth: 6",
"*scriptBox.scriptdone.marginHeight: 4",
"*scriptBox.scriptdone.rightAttachment: ATTACH_FORM",
"*scriptBox.scriptdone.bottomAttachment: ATTACH_FORM",
"*scriptBox.scriptdone.bottomOffset: 12",
"*scriptBox.sep.bottomAttachment: ATTACH_WIDGET",
"*scriptBox.sep.bottomWidget: scriptsend",
"*scriptBox.sep.leftAttachment: ATTACH_FORM",
"*scriptBox.sep.rightAttachment: ATTACH_FORM",
"*scriptBox.scriptText.editMode: MULTI_LINE_EDIT",
"*scriptBox.scriptText.width: 400",
"*scriptBox.scriptText.height: 300",
"*scriptBox.scriptText.topAttachment: ATTACH_FORM",
"*scriptBox.scriptText.leftAttachment: ATTACH_FORM",
"*scriptBox.scriptText.rightAttachment: ATTACH_FORM",
"*scriptBox.scriptText.bottomAttachment: ATTACH_WIDGET",
"*scriptBox.scriptText.bottomWidget: sep",
"*topform.horizontalSpacing: 0",
"*topform.verticalSpacing: 0",
"*menuBar*background: grey65",
"*menuBar.topAttachment: ATTACH_FORM",
"*menuBar.leftAttachment: ATTACH_FORM",
"*menuBar.rightAttachment: ATTACH_FORM",
"*sepMenu.separatorType: Shadow_Etched_In",
"*control.labelString: File",
"*control.mnemonic: F",
"*controlMenu.open.labelString: Open Script...",
"*controlMenu.open.mnemonic: O",
"*controlMenu.save.labelString: Save Script...",
"*controlMenu.save.mnemonic: S",
"*controlMenu.script.labelString: Script",
"*controlMenu.script.mnemonic: c",
"*controlMenu.script.accelerator: Ctrl<Key>c:",
"*controlMenu.script.acceleratorText: ^c",
"*controlMenu.starticol.labelString: Edit Colormap",
"*controlMenu.starticol.mnemonic: E",
"*controlMenu.restart.labelString: Restart Raz",
"*controlMenu.restart.mnemonic: R",
"*controlMenu.quit.labelString: Exit",
"*controlMenu.quit.mnemonic: x",
"*controlMenu.quit.accelerator: Ctrl<Key>x:",
"*controlMenu.quit.acceleratorText: ^x",
"*image.labelString: Image",
"*image.mnemonic: I",
"*imageMenu.imgfile.labelString: Image Files...",
"*imageMenu.imgfile.mnemonic: I",
"*imageMenu.format.labelString: Format...",
"*imageMenu.format.mnemonic: F",
"*imageMenu.format.accelerator: Ctrl<Key>f:",
"*imageMenu.format.acceleratorText: ^f",
"*imageMenu.rawdisk.labelString: Raw Disk...",
"*imageMenu.rawdisk.mnemonic: R",
"*imageMenu.rawdisk.accelerator: Ctrl<Key>r:",
"*imageMenu.rawdisk.acceleratorText: ^r",
"*imageMenu.playraw.labelString: Play from Raw Disk",
"*imageMenu.playraw.indicatorType: One_Of_Many",
"*imageMenu.playraw.set: true",
"*imageMenu.playmem.labelString: Play from Memory",
"*imageMenu.playmem.indicatorType: One_Of_Many",
"*imageMenu.sourcefile.labelString: Source from File",
"*imageMenu.sourcefile.indicatorType: One_Of_Many",
"*imageMenu.sourcefile.set: true",
"*imageMenu.sourceraw.labelString: Source from Raw Disk",
"*imageMenu.sourceraw.indicatorType: One_Of_Many",
"*imageMenu.sourcetape.labelString: Source from Tape",
"*imageMenu.sourcetape.indicatorType: One_Of_Many",
"*mode.labelString: Mode",
"*mode.mnemonic: M",
"*modeMenu.zoom.labelString: Zoom Window",
"*modeMenu.zoom.mnemonic: Z",
"*modeMenu.zoom.accelerator: Ctrl<Key>z:",
"*modeMenu.zoom.acceleratorText: ^z",
"*modeMenu.timer.labelString: Frame Timer",
"*modeMenu.timer.mnemonic: T",
"*modeMenu.timer.accelerator: Ctrl<Key>t:",
"*modeMenu.timer.acceleratorText: ^t",
"*modeMenu.doublebuf.labelString: Double Buffered",
"*modeMenu.doublebuf.mnemonic: D",
"*modeMenu.doublebuf.set: on",
"*modeMenu.reverse.labelString: Reverse at End",
"*modeMenu.reverse.mnemonic: R",
"*ahpcrc.labelString: AHPCRC Raster Play",
"*ahpcrc.foreground: red4",
"*ahpcrc.fontList: -*-times-bold-r-*-*-*-140-*-*-*-*-*-*",
"*ahpcrc.topAttachment: ATTACH_WIDGET",
"*ahpcrc.topWidget: menuBar",
"*ahpcrc.topOffset: 6",
"*ahpcrc.leftAttachment: ATTACH_FORM",
"*ahpcrc.leftOffset: 6",
"*ahpcrc.rightAttachment: ATTACH_FORM",
"*ahpcrc.rightOffset: 6",
"*ahpcrc.highlightThickness: 0",
"*playFrame.shadowType: Shadow_Etched_In",
"*playFrame.topAttachment: Attach_Widget",
"*playFrame.topWidget: ahpcrc",
"*playFrame.topOffset: 6",
"*playFrame.leftAttachment: Attach_Form",
"*playFrame.leftOffset: 6",
"*playFrame.rightAttachment: Attach_Form",
"*playFrame.rightOffset: 6",
"*playFrame.marginWidth: 4",
"*playFrame.marginHeight: 4",
"*anchor.separatorType: No_Line",
"*anchor.leftAttachment: Attach_Position",
"*anchor.leftPosition: 50",
"*buttonBox.orientation: Horizontal",
"*buttonBox.spacing: 2",
"*buttonBox.topAttachment: Attach_Form",
"*buttonBox.leftAttachment: Attach_Opposite_Widget",
"*buttonBox.leftWidget: anchor",
"*buttonBox.leftOffset: -112",
"*buttonBox.XmArrowButton.shadowThickness: 0",
"*buttonBox*playRev.arrowDirection: Arrow_Left",
"*buttonBox*playRev.width: 50",
"*buttonBox*jogRev.arrowDirection: Arrow_Left",
"*buttonBox*jogRev.width: 30",
"*buttonBox*stop.labelType: pixmap",
"*buttonBox*stop.foreground: #c00",
"*buttonBox*stop.fillOnArm: false",
"*buttonBox*stop.marginHeight: 4",
"*buttonBox*stop.marginWidth: 4",
"*buttonBox*jogFwd.arrowDirection: Arrow_Right",
"*buttonBox*jogFwd.width: 30",
"*buttonBox*playFwd.arrowDirection: Arrow_Right",
"*buttonBox*playFwd.width: 50",
"*status.labelString: All Quiet",
"*status.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*status.alignment: Alignment_Center",
"*status.topAttachment: Attach_Widget",
"*status.topWidget: playFrame",
"*status.topOffset: 6",
"*status.leftAttachment: Attach_Form",
"*status.leftOffset: 6",
"*status.rightAttachment: Attach_Form",
"*status.rightOffset: 6",
"*sliderFrame.shadowType: Shadow_Etched_In",
"*sliderFrame.topAttachment: Attach_Widget",
"*sliderFrame.topWidget: status",
"*sliderFrame.topOffset: 6",
"*sliderFrame.leftAttachment: Attach_Form",
"*sliderFrame.leftOffset: 6",
"*sliderFrame.rightAttachment: Attach_Form",
"*sliderFrame.rightOffset: 6",
"*sliderFrame.bottomAttachment: Attach_Form",
"*sliderFrame.bottomOffset: 6",
"*sliderFrame.marginWidth: 4",
"*sliderFrame.marginHeight: 4",
"*slider.width: 300",
"*slider.XmLabel.leftAttachment: Attach_Form",
"*slider.XmLabel.alignment: Alignment_Beginning",
"*slider.XmLabel.topAttachment: Attach_Widget",
"*slider.XmLabel.topOffset: 15",
"*slider.XmLabel.width: 50",
"*slider.frameLabel.labelString: Frame",
"*slider.frameLabel.topAttachment: Attach_Form",
"*slider.speedLabel.labelString: Speed",
"*slider.speedLabel.topWidget: frameLabel",
"*slider.sizeLabel.labelString: Size",
"*slider.sizeLabel.topWidget: speedLabel",
"*slider.borderLabel.labelString: Border",
"*slider.borderLabel.topWidget: sizeLabel",
"*slider.zoomLabel.labelString: Zoom",
"*slider.zoomLabel.topWidget: borderLabel",
"*slider.XmScale.showValue: on",
"*slider.XmScale.bottomAttachment: Attach_Opposite_Widget",
"*slider.XmScale.rightAttachment: Attach_Form",
"*slider.XmScale.leftAttachment: Attach_Form",
"*slider.XmScale.leftOffset: 56",
"*slider.XmScale.orientation: Horizontal",
"*slider.XmScale.fontList: 7x13",
"*slider.frame.bottomWidget: frameLabel",
"*slider.frame.minimum: 1",
"*slider.frame.maximum: 100",
"*slider.speed.bottomWidget: speedLabel",
"*slider.speed.decimalPoints: 1",
"*slider.speed.minimum: 10",
"*slider.speed.maximum: 300",
"*slider.speed.value: 300",
"*slider.speed.scaleMultiple: 10",
"*slider.size.bottomWidget: sizeLabel",
"*slider.size.decimalPoints: 1",
"*slider.size.minimum: 1",
"*slider.size.maximum: 50",
"*slider.size.value: 10",
"*slider.size.scaleMultiple: 5",
"*slider.border.bottomWidget: borderLabel",
"*slider.border.minimum: 0",
"*slider.border.maximum: 256",
"*slider.zoomsize.bottomWidget: zoomLabel",
"*slider.zoomsize.decimalPoints: 1",
"*slider.zoomsize.minimum: 1",
"*slider.zoomsize.maximum: 100",
"*slider.zoomsize.value: 40",
"*slider.zoomsize.scaleMultiple: 10",
