
/* Copyright (C) 1992  AHPCRC, Univeristy of Minnesota
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named 'Copying'; if not, write to
 * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139.
 */

/* Author:
 *	Ken Chin-Purcell (ken@ahpcrc.umn.edu)
 *	Army High Performance Computing Research Center (AHPCRC)
 *	Univeristy of Minnesota
 *
 * $Header: /usr/people/ken/gvl/icol/RCS/setup.h,v 2.1 92/10/19 16:59:37 ken Exp $
 *
 * $Log:	setup.h,v $
 * Revision 2.1  92/10/19  16:59:37  ken
 * *** empty log message ***
 * 
 * Revision 1.1  92/10/19  16:58:15  ken
 * Initial revision
 * 
 */

/* Prototypes */

/* setup.c */
void	CreateBitmap (Widget w, int id);
void	RemoveHelp (Widget w);
void	RemoveCancel (Widget w);
void	SetupIcol (unsigned int *argc, char *argv []);
