"*device.value: /dev/anim0",
"*saveBGR: true",
"*background: grey75",
"*allowShellResize: true",
"*fontList: -*-helvetica-medium-r-normal--*-100-*-*-*-*-*-*",
"*XmPushButton.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmPushButtonGadget.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*menuBar*fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmToggleButton.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmToggleButton.selectColor: red",
"*XmToggleButton.visibleWhenOff: true",
"*XmToggleButtonGadget.fontList: -*-helvetica-bold-r-normal--*-100-*-*-*-*-*-*",
"*XmToggleButtonGadget.selectColor: red",
"*XmToggleButtonGadget.visibleWhenOff: true",
"*fsBox.dialogTitle: Bob File Select",
"*fsBox*XmText.width: 400",
"*rendBox.dialogTitle: Remote Render",
"*rendBox.horizontalSpacing: 6",
"*rendBox.verticalSpacing: 6",
"*rendBox.autoUnmanage: False",
"*rendBox.dorend.labelString: Render",
"*rendBox.dorend.showAsDefault: true",
"*rendBox.dorend.marginWidth: 6",
"*rendBox.dorend.marginHeight: 4",
"*rendBox.dorend.leftAttachment: ATTACH_FORM",
"*rendBox.dorend.bottomAttachment: ATTACH_FORM",
"*rendBox.sep.bottomAttachment: ATTACH_WIDGET",
"*rendBox.sep.bottomWidget: dorend",
"*rendBox.sep.leftAttachment: ATTACH_FORM",
"*rendBox.sep.rightAttachment: ATTACH_FORM",
"*rendBox.hostTitle.labelString: Remote Host:",
"*rendBox.hostTitle.topAttachment: ATTACH_FORM",
"*rendBox.hostTitle.leftAttachment: ATTACH_FORM",
"*rendBox.hostTitle.rightAttachment: ATTACH_FORM",
"*rendBox.hostTitle.alignment: Alignment_Beginning",
"*rendBox.rendHost.fontList: fixed",
"*rendBox.rendHost.topAttachment: ATTACH_WIDGET",
"*rendBox.rendHost.topWidget: hostTitle",
"*rendBox.rendHost.leftAttachment: ATTACH_FORM",
"*rendBox.rendHost.rightAttachment: ATTACH_FORM",
"*rendBox.textTitle.labelString: Rendering Text:",
"*rendBox.textTitle.topAttachment: ATTACH_WIDGET",
"*rendBox.textTitle.topWidget: rendHost",
"*rendBox.textTitle.leftAttachment: ATTACH_FORM",
"*rendBox.textTitle.rightAttachment: ATTACH_FORM",
"*rendBox.textTitle.alignment: Alignment_Beginning",
"*rendBox.rendText.editMode: MULTI_LINE_EDIT",
"*rendBox.rendText.fontList: fixed",
"*rendBox.rendText.width: 400",
"*rendBox.rendText.height: 400",
"*rendBox.rendText.topAttachment: ATTACH_WIDGET",
"*rendBox.rendText.topWidget: textTitle",
"*rendBox.rendText.leftAttachment: ATTACH_FORM",
"*rendBox.rendText.rightAttachment: ATTACH_FORM",
"*rendBox.rendText.bottomAttachment: ATTACH_WIDGET",
"*rendBox.rendText.bottomWidget: sep",
"*rendBox.rendHost.value: e2.msc.edu -l mxm1901",
"*rendBox.rendText.value: rsh $RENDHOST 'cat >> /tmp/cmds.vr' <<DONE \\nimagesize    $IXSIZE $IYSIZE	\\ncolortable   /tmp/ctab.vr      	\\nopacitytable /tmp/otab.vr 	\\nviewmode     bearing		\\nelev         $ELEV		\\nazim         $AZIM		\\nfov          45			\\ninters       0			\\nrender				\\n*dimDialog.dialogTitle:		Bob Data Dimensions",
"*dimDialog.dialogStyle: Dialog_Modeless",
"*dimDialog.defaultPosition: False",
"*finderFrame.shadowType: Shadow_In",
"*finderFrame.topAttachment: Attach_Form",
"*finderFrame.leftAttachment: Attach_Form",
"*finderFrame.bottomAttachment: Attach_Form",
"*finderFrame.rightAttachment: Attach_Form",
"*finderFrame.rightOffset: 560",
"*finder.width: 250",
"*finder.height: 250",
"*dimBox.horizontalSpacing: 4",
"*dimBox.verticalSpacing: 4",
"*dimBox.topAttachment: Attach_Form",
"*dimBox.rightAttachment: Attach_Form",
"*dimBox.bottomAttachment: Attach_Form",
"*dimBox.leftAttachment: Attach_Widget",
"*dimBox.leftWidget: finderFrame",
"*dimBox.XmScale.orientation: Horizontal",
"*dimBox.XmScale.showValue: on",
"*dimBox.XmScale.topAttachment: Attach_Position",
"*dimBox.XmScale.leftAttachment: Attach_Form",
"*dimBox.XmScale.rightAttachment: Attach_Form",
"*dimBox.XmScale.rightPosition: 50",
"*dimBox.XmScale.leftPosition: 50",
"*dimBox*XmLabelGadget.alignment: Alignment_Beginning",
"*dimBox*XmLabelGadget.width: 200",
"*dimBox.xcenter.topPosition: 0",
"*dimBox.xcenter.rightAttachment: Attach_Position",
"*dimBox.xcenter.rightOffset: 2",
"*dimBox.xcenter.titleString: X Center",
"*dimBox.xcenter.minimum: 1",
"*dimBox.ycenter.topPosition: 25",
"*dimBox.ycenter.rightAttachment: Attach_Position",
"*dimBox.ycenter.rightOffset: 2",
"*dimBox.ycenter.titleString: Y Center",
"*dimBox.ycenter.minimum: 1",
"*dimBox.zcenter.topPosition: 50",
"*dimBox.zcenter.rightAttachment: Attach_Position",
"*dimBox.zcenter.rightOffset: 2",
"*dimBox.zcenter.titleString: Z Center",
"*dimBox.zcenter.minimum: 1",
"*dimBox.stride.topPosition: 75",
"*dimBox.stride.rightAttachment: Attach_Position",
"*dimBox.stride.rightOffset: 2",
"*dimBox.stride.titleString: Stride",
"*dimBox.stride.minimum: 1",
"*dimBox.stride.maximum: 16",
"*dimBox.stride.value: 1",
"*dimBox.xsize.topPosition: 0",
"*dimBox.xsize.leftAttachment: Attach_Position",
"*dimBox.xsize.leftOffset: 2",
"*dimBox.xsize.titleString: X Size",
"*dimBox.xsize.value: 32",
"*dimBox.xsize.minimum: 2",
"*dimBox.ysize.topPosition: 25",
"*dimBox.ysize.leftAttachment: Attach_Position",
"*dimBox.ysize.leftOffset: 2",
"*dimBox.ysize.titleString: Y Size",
"*dimBox.ysize.value: 32",
"*dimBox.ysize.minimum: 2",
"*dimBox.zsize.topPosition: 50",
"*dimBox.zsize.leftAttachment: Attach_Position",
"*dimBox.zsize.leftOffset: 2",
"*dimBox.zsize.titleString: Z Size",
"*dimBox.zsize.value: 32",
"*dimBox.zsize.minimum: 2",
"*dimBox.msize.topPosition: 75",
"*dimBox.msize.leftAttachment: Attach_Position",
"*dimBox.msize.leftOffset: 2",
"*dimBox.msize.titleString: Max Size",
"*dimBox.msize.value: 32",
"*dimBox.msize.minimum: 2",
"*anDialog.dialogTitle: Bob Animation",
"*anDialog.dialogStyle: Dialog_Modeless",
"*anDialog.x: 20",
"*anDialog.y: 200",
"*anDialog.defaultPosition: False",
"*anDialog.autoUnmanage: False",
"*anDialog.horizontalSpacing: 3",
"*anDialog.verticalSpacing: 2",
"*anDialog.XmTextField.background: grey85",
"*anDialog.XmTextField.columns: 25",
"*anDialog.XmTextField.leftAttachment: Attach_Form",
"*anDialog.XmTextField.leftOffset: 100",
"*anDialog.XmTextField.rightAttachment: Attach_Form",
"*anDialog.XmTextField.topAttachment: Attach_Widget",
"*anDialog.XmLabel.leftAttachment: Attach_Form",
"*anDialog.XmLabel.bottomAttachment: Attach_Opposite_Widget",
"*anDialog.XmLabel.bottomOffset: 4",
"*anDialog.axisLabel.labelString: Axis (x y z)",
"*anDialog.axisLabel.bottomWidget: axis",
"*anDialog.axis.topAttachment: Attach_Form",
"*anDialog.axis.topOffset: 6",
"*anDialog.axis.value: 0 1 0",
"*anDialog.volumeLabel.labelString: Volume...",
"*anDialog.volumeLabel.bottomWidget: axisX",
"*anDialog.axisX.labelString: X axis",
"*anDialog.axisX.fontList: -*-helvetica-medium-r-normal--*-100-*-*-*-*-*-*",
"*anDialog.axisX.topAttachment: Attach_Widget",
"*anDialog.axisX.topWidget: axis",
"*anDialog.axisX.leftAttachment: Attach_Form",
"*anDialog.axisX.leftOffset: 100",
"*anDialog.axisY.labelString: Y axis",
"*anDialog.axisY.fontList: -*-helvetica-medium-r-normal--*-100-*-*-*-*-*-*",
"*anDialog.axisY.topAttachment: Attach_Widget",
"*anDialog.axisY.topWidget: axis",
"*anDialog.axisY.leftAttachment: Attach_Widget",
"*anDialog.axisY.leftWidget: axisX",
"*anDialog.axisZ.labelString: Z axis",
"*anDialog.axisZ.fontList: -*-helvetica-medium-r-normal--*-100-*-*-*-*-*-*",
"*anDialog.axisZ.topAttachment: Attach_Widget",
"*anDialog.axisZ.topWidget: axis",
"*anDialog.axisZ.leftAttachment: Attach_Widget",
"*anDialog.axisZ.leftWidget: axisY",
"*anDialog.incLabel.labelString: Rotation (deg)",
"*anDialog.incLabel.bottomWidget: increment",
"*anDialog.increment.topWidget: axisX",
"*anDialog.increment.value: 2",
"*anDialog.fincLabel.labelString: Frame step",
"*anDialog.fincLabel.bottomWidget: frameinc",
"*anDialog.frameinc.topWidget: increment",
"*anDialog.frameinc.value: 1",
"*anDialog.ansep3.separatorType: Shadow_Etched_In",
"*anDialog.ansep3.orientation: Horizontal",
"*anDialog.ansep3.rightAttachment: Attach_Form",
"*anDialog.ansep3.leftAttachment: Attach_Form",
"*anDialog.ansep3.topAttachment: Attach_Widget",
"*anDialog.ansep3.topWidget: frameinc",
"*anDialog.ansep3.topOffset: 10",
"*anDialog.anSave.labelString: Save Animation:",
"*anDialog.anSave.indicatorType: N_Of_Many",
"*anDialog.anSave.set: on",
"*anDialog.anSave.topAttachment: Attach_Widget",
"*anDialog.anSave.topWidget: ansep3",
"*anDialog.anSave.topOffset: 4",
"*anDialog.anSave.leftAttachment: Attach_Form",
"*anDialog.nstepLabel.labelString: Steps",
"*anDialog.nstepLabel.bottomWidget: nstep",
"*anDialog.nstep.topWidget: anSave",
"*anDialog.nstep.value: 180",
"*anDialog.vsizeLabel.labelString: Size (x y)",
"*anDialog.vsizeLabel.bottomWidget: vsize",
"*anDialog.vsize.topWidget: nstep",
"*anDialog.vsize.value: 512 512",
"*anDialog.deviceLabel.labelString: File",
"*anDialog.deviceLabel.bottomWidget: device",
"*anDialog.device.topWidget: vsize",
"*anDialog.offsetLabel.labelString: Offset (bytes)",
"*anDialog.offsetLabel.bottomWidget: offset",
"*anDialog.offset.topWidget: device",
"*anDialog.offset.value: 0",
"*anDialog.anFrame.labelString: Not animating",
"*anDialog.anFrame.bottomAttachment: Attach_None",
"*anDialog.anFrame.topAttachment: Attach_Widget",
"*anDialog.anFrame.topWidget: offset",
"*anDialog.anFrame.topOffset: 10",
"*anDialog.ansep.separatorType: Shadow_Etched_In",
"*anDialog.ansep.orientation: Horizontal",
"*anDialog.ansep.rightAttachment: Attach_Form",
"*anDialog.ansep.leftAttachment: Attach_Form",
"*anDialog.ansep.topAttachment: Attach_Widget",
"*anDialog.ansep.topWidget: anFrame",
"*anDialog.ansep.topOffset: 10",
"*anDialog.anGo.labelString: Go!",
"*anDialog.anGo.width: 70",
"*anDialog.anGo.topAttachment: Attach_Widget",
"*anDialog.anGo.topWidget: ansep",
"*anDialog.anGo.topOffset: 8",
"*anDialog.anGo.leftAttachment: Attach_Form",
"*anDialog.anGo.leftOffset: 30",
"*anDialog.anDone.labelString: Done",
"*anDialog.anDone.width: 70",
"*anDialog.anDone.topAttachment: Attach_Widget",
"*anDialog.anDone.topWidget: ansep",
"*anDialog.anDone.topOffset: 8",
"*anDialog.anDone.rightAttachment: Attach_Form",
"*anDialog.anDone.rightOffset: 30",
"*anDialog.ansep2.separatorType: No_Line",
"*anDialog.ansep2.orientation: Horizontal",
"*anDialog.ansep2.rightAttachment: Attach_Form",
"*anDialog.ansep2.leftAttachment: Attach_Form",
"*anDialog.ansep2.topAttachment: Attach_Widget",
"*anDialog.ansep2.topWidget: anGo",
"*anDialog.ansep2.topOffset: 7",
"*copyright.dialogTitle: All About Bob",
"*copyright*fontList: -*-rock-medium-r-normal--*-180-*-*-*-*-*-*",
"*copyright*Background: tan4",
"*copyright*Foreground: white",
"*copyright.okLabelString: Enough About Bob",
"*copyright.cancelLabelString: Mail Comment",
"*copyright.messageString: \\nBob  -  Brick of Bytes\\n   Version 1.2\\n      Copyright 1992 University of Minnesota\\n      Distributed under the GNU General Public License\\n\\nFrom:\\n   Graphics and Visualization Lab,\\n      Army High Performance Computing Research Center,\\n      University of Minnesota\\n\\nWritten by:\\n   Ken Chin-Purcell\\n\\nThanks to:\\n    Paul Morin, Dave Porter, and the MSCI graphics group\\n",
"*bugBox.dialogTitle: Bob Comment",
"*bugBox.horizontalSpacing: 6",
"*bugBox.verticalSpacing: 6",
"*bugBox.send.labelString: Send",
"*bugBox.send.showAsDefault: true",
"*bugBox.send.marginWidth: 6",
"*bugBox.send.marginHeight: 4",
"*bugBox.send.leftAttachment: ATTACH_FORM",
"*bugBox.send.bottomAttachment: ATTACH_FORM",
"*bugBox.cancel.labelString: Cancel",
"*bugBox.cancel.marginWidth: 6",
"*bugBox.cancel.marginHeight: 4",
"*bugBox.cancel.rightAttachment: ATTACH_FORM",
"*bugBox.cancel.bottomAttachment: ATTACH_FORM",
"*bugBox.cancel.bottomOffset: 12",
"*bugBox.sep.bottomAttachment: ATTACH_WIDGET",
"*bugBox.sep.bottomWidget: send",
"*bugBox.sep.leftAttachment: ATTACH_FORM",
"*bugBox.sep.rightAttachment: ATTACH_FORM",
"*bugBox.bugText.value: My comment on Bob is...\\n\\n",
"*bugBox.bugText.editMode: MULTI_LINE_EDIT",
"*bugBox.bugText.width: 400",
"*bugBox.bugText.height: 300",
"*bugBox.bugText.topAttachment: ATTACH_FORM",
"*bugBox.bugText.leftAttachment: ATTACH_FORM",
"*bugBox.bugText.rightAttachment: ATTACH_FORM",
"*bugBox.bugText.bottomAttachment: ATTACH_WIDGET",
"*bugBox.bugText.bottomWidget: sep",
"*topform.horizontalSpacing: 0",
"*topform.verticalSpacing: 0",
"*voxframe.shadowType: Shadow_In",
"*voxframe.topAttachment: Attach_Form",
"*voxframe.leftAttachment: Attach_Form",
"*voxframe.bottomAttachment: Attach_Form",
"*voxframe.rightAttachment: Attach_Form",
"*voxframe.rightOffset: 300",
"*vox.width: 512",
"*vox.height: 512",
"*stereoVox.x: 0",
"*stereoVox.y: 0",
"*stereoVox.width: 1280",
"*stereoVox.height: 1024",
"*stereoVox.borderWidth: 0",
"*stereoShell.x: 0",
"*stereoShell.y: 0",
"*stereoShell.borderWidth: 0",
"*menuBar*background: grey65",
"*menuBar.topAttachment: Attach_Form",
"*menuBar.rightAttachment: Attach_Form",
"*menuBar.leftAttachment: Attach_Widget",
"*menuBar.leftWidget: voxframe",
"*sepMenu.separatorType: Shadow_Etched_In",
"*control.labelString: File",
"*control.mnemonic: F",
"*controlMenu.open.labelString: Open...",
"*controlMenu.open.mnemonic: O",
"*controlMenu.save.labelString: Save...",
"*controlMenu.save.mnemonic: S",
"*controlMenu.startcmap.labelString: Color Map Editor",
"*controlMenu.startcmap.mnemonic: C",
"*controlMenu.startamap.labelString: Alpha Map Editor",
"*controlMenu.startamap.mnemonic: l",
"*controlMenu.animate.labelString: Animate...",
"*controlMenu.animate.mnemonic: A",
"*controlMenu.openDim.labelString: Dimension...",
"*controlMenu.openDim.mnemonic: D",
"*controlMenu.remote.labelString: Remote...",
"*controlMenu.remote.mnemonic: R",
"*controlMenu.quit.labelString: Exit",
"*controlMenu.quit.mnemonic: x",
"*controlMenu.quit.accelerator: Ctrl<Key>x:",
"*controlMenu.quit.acceleratorText: ^x",
"*view.labelString: View",
"*view.mnemonic: V",
"*viewMenu.stereo.labelString: Stereo",
"*viewMenu.stereo.mnemonic: t",
"*viewMenu.home.labelString: Home View",
"*viewMenu.home.mnemonic: H",
"*viewMenu.home.accelerator: Ctrl<Key>space:",
"*viewMenu.home.acceleratorText: ^sp",
"*viewMenu.flipx.labelString: Flip X",
"*viewMenu.flipx.mnemonic: X",
"*viewMenu.flipy.labelString: Flip Y",
"*viewMenu.flipy.mnemonic: Y",
"*viewMenu.flipz.labelString: Flip Z",
"*viewMenu.flipz.mnemonic: Z",
"*viewMenu.dbuffer.labelString: Double Buffer",
"*viewMenu.dbuffer.mnemonic: D",
"*viewMenu.dbuffer.accelerator: Ctrl<Key>d:",
"*viewMenu.dbuffer.acceleratorText: ^d",
"*viewMenu.dbuffer.indicatorType: N_Of_Many",
"*viewMenu.dbuffer.set: on",
"*viewMenu.fog.labelString: Fog",
"*viewMenu.fog.mnemonic: F",
"*viewMenu.fog.indicatorType: N_Of_Many",
"*viewMenu.fog.set: off",
"*viewMenu.brightness.labelString: Auto Alpha",
"*viewMenu.brightness.mnemonic: A",
"*viewMenu.brightness.indicatorType: N_Of_Many",
"*viewMenu.brightness.set: on",
"*viewMenu.softEdge.labelString: Soft Edges",
"*viewMenu.softEdge.mnemonic: S",
"*viewMenu.softEdge.indicatorType: N_Of_Many",
"*viewMenu.softEdge.set: off",
"*viewMenu.bounds.labelString: Bounding Box",
"*viewMenu.bounds.mnemonic: B",
"*viewMenu.bounds.indicatorType: N_Of_Many",
"*viewMenu.bounds.set: on",
"*viewMenu.bounds.accelerator: Ctrl<Key>b:",
"*viewMenu.bounds.acceleratorText: ^b",
"*viewMenu.annotate.labelString: Annotation",
"*viewMenu.annotate.mnemonic: A",
"*viewMenu.annotate.indicatorType: N_Of_Many",
"*viewMenu.annotate.set: on",
"*viewMenu.annotate.accelerator: Ctrl<Key>n:",
"*viewMenu.annotate.acceleratorText: ^n",
"*viewMenu.colorBar.labelString: Color Bar",
"*viewMenu.colorBar.mnemonic: C",
"*viewMenu.colorBar.indicatorType: N_Of_Many",
"*viewMenu.colorBar.set: off",
"*viewMenu.colorBar.accelerator: Ctrl<Key>c:",
"*viewMenu.colorBar.acceleratorText: ^c",
"*viewMenu.point.labelString: Points",
"*viewMenu.point.mnemonic: P",
"*viewMenu.point.indicatorType: N_Of_Many",
"*viewMenu.point.set: off",
"*viewMenu.point.accelerator: Ctrl<Key>p:",
"*viewMenu.point.acceleratorText: ^p",
"*mode.labelString: Mode",
"*mode.mnemonic: M",
"*modeMenu.dbuffer.labelString: Double Buffer",
"*modeMenu.dbuffer.mnemonic: D",
"*modeMenu.dbuffer.accelerator: Ctrl<Key>d:",
"*modeMenu.dbuffer.acceleratorText: ^d",
"*modeMenu.dbuffer.indicatorType: N_Of_Many",
"*modeMenu.dbuffer.set: on",
"*modeMenu.brightness.labelString: Auto Alpha",
"*modeMenu.brightness.mnemonic: A",
"*modeMenu.brightness.indicatorType: N_Of_Many",
"*modeMenu.brightness.set: on",
"*modeMenu.softEdge.labelString: Soft Edges",
"*modeMenu.softEdge.mnemonic: S",
"*modeMenu.softEdge.indicatorType: N_Of_Many",
"*modeMenu.softEdge.set: off",
"*modeMenu.interpolate.labelString: Interpolate",
"*modeMenu.interpolate.mnemonic: I",
"*modeMenu.interpolate.indicatorType: N_Of_Many",
"*modeMenu.interpolate.set: off",
"*modeMenu.maxval.labelString: Maximum Value",
"*modeMenu.maxval.mnemonic: M",
"*modeMenu.maxval.indicatorType: N_Of_Many",
"*modeMenu.maxval.set: off",
"*modeMenu.maxval.accelerator: Ctrl<Key>m:",
"*modeMenu.maxval.acceleratorText: ^m",
"*dimension.labelString: Dimension",
"*dimension.mnemonic: D",
"*dimMenu.dactive.labelString: Active",
"*dimMenu.dactive.mnemonic: A",
"*dimMenu.dactive.indicatorType: N_Of_Many",
"*dimMenu.dactive.set: on",
"*dimMenu.dactive.accelerator: Ctrl<Key>t:",
"*dimMenu.dactive.acceleratorText: ^t",
"*dimMenu.dapply.labelString: Apply",
"*dimMenu.dapply.mnemonic: p",
"*dimMenu.dapply.accelerator: Ctrl<Key>a:",
"*dimMenu.dapply.acceleratorText: ^a",
"*dimMenu.dadd.labelString: Add Entry",
"*dimMenu.dadd.mnemonic: E",
"*dimMenu.dadd.accelerator: Ctrl<Key>i:",
"*dimMenu.dadd.acceleratorText: ^i",
"*dimMenu.ddelete.labelString: Delete Entry",
"*dimMenu.ddelete.mnemonic: D",
"*dimMenu.ddelete.accelerator: Ctrl<Key>k:",
"*dimMenu.ddelete.acceleratorText: ^k",
"*infoform.horizontalSpacing: 4",
"*infoform.verticalSpacing: 4",
"*infoform.topAttachment: Attach_Widget",
"*infoform.topWidget: menuBar",
"*infoform.rightAttachment: Attach_Form",
"*infoform.leftAttachment: Attach_Widget",
"*infoform.leftWidget: voxframe",
"*infoform.bottomAttachment: Attach_Form",
"*infoform.resizable: false",
"*ahpcrc.labelString: AHPCRC Volume Render",
"*ahpcrc.foreground: red4",
"*ahpcrc.fontList: -*-times-bold-r-*-*-*-140-*-*-*-*-*-*",
"*ahpcrc.topAttachment: Attach_Form",
"*ahpcrc.leftAttachment: Attach_Form",
"*ahpcrc.rightAttachment: Attach_Form",
"*ahpcrc.highlightThickness: 0",
"*alpha.showValue: on",
"*alpha.topAttachment: Attach_Widget",
"*alpha.topWidget: ahpcrc",
"*alpha.rightAttachment: Attach_Form",
"*alpha.leftAttachment: Attach_Form",
"*alpha.orientation: Horizontal",
"*alpha.minimum: 1",
"*alpha.maximum: 200",
"*alpha.decimalPoints: 2",
"*alpha.value: 100",
"*alpha.titleString: Alpha Attenuation",
"*frame.showValue: on",
"*frame.topAttachment: Attach_Widget",
"*frame.topWidget: alpha",
"*frame.topOffset: -4",
"*frame.rightAttachment: Attach_Form",
"*frame.leftAttachment: Attach_Form",
"*frame.orientation: Horizontal",
"*frame.titleString: Frame",
"*frame.minimum: 1",
"*frame.value: 1",
"*filelistSW.topAttachment: Attach_Widget",
"*filelistSW.topWidget: frame",
"*filelistSW.rightAttachment: Attach_Form",
"*filelistSW.leftAttachment: Attach_Form",
"*filelistSW.bottomAttachment: Attach_Position",
"*filelistSW.bottomPosition: 75",
"*filelist.scrollBarDisplayPolicy: Static",
"*filelist.selectionPolicy: Single_Select",
"*filelist.listSizePolicy: Constant",
"*dimlistSW.topAttachment: Attach_Widget",
"*dimlistSW.topWidget: filelistSW",
"*dimlistSW.rightAttachment: Attach_Form",
"*dimlistSW.leftAttachment: Attach_Form",
"*dimlistSW.bottomAttachment: Attach_Form",
"*dimlist.scrollBarDisplayPolicy: Static",
"*dimlist.selectionPolicy: Single_Select",
"*dimlist.listSizePolicy: Constant",
